<?php

namespace MailOptin\MailChimpConnect;

use Mailchimp\MailchimpCampaigns;
use Mailchimp\MailchimpLists;
use MailOptin\Core\Connections\AbstractConnect;
use MailOptin\Core\PluginSettings\Connections;
use MailOptin\Core\PluginSettings\Settings;

class AbstractMailChimpConnect extends AbstractConnect
{
    /** @var \MailOptin\Core\PluginSettings\Settings */
    protected $plugin_settings;

    /** @var \MailOptin\Core\PluginSettings\Connections */
    protected $connections_settings;

    public function __construct()
    {
        $this->plugin_settings = Settings::instance();
        $this->connections_settings = Connections::instance();
    }

    /**
     * Return instance of MailChimp list class.
     *
     * @throws \Exception
     *
     * @return MailchimpLists
     */
    public function mc_list_instance()
    {
        $api_key = $this->connections_settings->mailchimp_api_key();

        if (empty($api_key)) {
            throw new \Exception(__('MailChimp API key not found.', 'mailoptin'));
        }

        return new MailchimpLists($api_key);
    }

    /**
     * Return instance of MailChimp campaign class.
     *
     * @throws \Exception
     *
     * @return MailchimpCampaigns
     */
    public function mc_campaign_instance()
    {
        $api_key = $this->connections_settings->mailchimp_api_key();

        if (empty($api_key)) {
            throw new \Exception(__('MailChimp API key not found.', 'mailoptin'));
        }

        return new MailchimpCampaigns($api_key);
    }
}