<?php

namespace MailOptin\MailChimpConnect;

use MailOptin\Core\Repositories\OptinCampaignsRepository;

class Subscription extends AbstractMailChimpConnect
{
    protected $email;
    protected $name;
    protected $list_id;
    protected $extras;

    public function __construct($email, $name, $list_id, $extras)
    {
        $this->email = $email;
        $this->name = $name;
        $this->list_id = $list_id;
        $this->extras = $extras;

        parent::__construct();
    }

    /**
     * True if optin type is double optin is not disabled.
     *
     * @return bool
     */
    public function is_double_optin()
    {
        $optin_campaign_id = absint($this->extras['optin_campaign_id']);

        $setting = OptinCampaignsRepository::get_customizer_value($optin_campaign_id, 'MailChimpConnect_disable_double_optin');

        return $setting !== true;
    }

    /**
     * @return mixed
     */
    public function subscribe()
    {
        try {
            $name_split = self::get_first_last_names($this->name);

            $optin_status = $this->is_double_optin() ? 'pending' : 'subscribed';

            $parameters = [
                'merge_fields' => array_filter(
                    [
                        'FNAME' => $name_split[0],
                        'LNAME' => $name_split[1]
                    ],
                    'is_string'
                ),
                'status' => $optin_status
            ];

            $response = $this->mc_list_instance()->addMember($this->list_id, $this->email, $parameters);

            if (is_object($response) && 'subscribed' == $response->status) {
                return parent::ajax_success();
            }

            return parent::ajax_failure(__('There was an error adding the contact to MailChimp. Please try again.', 'mailoptin'));

        } catch (\Exception $e) {
            // if already subscribed, return success.
            if (strpos($e->getMessage(), 'Member Exists')) {
                return parent::ajax_success();
            }

            self::save_optin_error_log($e->getCode() . ': ' . $e->getMessage(), 'mailchimp');
            return parent::ajax_failure(__('There was an error adding the contact to MailChimp. Please try again.', 'mailoptin'));
        }
    }
}