<?php

namespace MailOptin\RegisteredUsersConnect;

use MailOptin\Core\Connections\AbstractConnect;
use MailOptin\Core\Connections\ConnectionInterface;

class Connect extends AbstractConnect implements ConnectionInterface
{
    /**
     * @var WP_Mail_BG_Process
     */
    public $bg_process_instance;

    /**
     * @var string key of connection service. its important all connection name ends with "Connect"
     */
    public static $connectionName = 'RegisteredUsersConnect';

    public function __construct()
    {
        add_filter('mailoptin_registered_connections', array($this, 'register_connection'));

        add_action('plugins_loaded', array($this, 'init'));
    }

    public function init()
    {
        $this->bg_process_instance = new WP_Mail_BG_Process();
    }

    /**
     * Register Sendy Connection.
     *
     * @param array $connections
     *
     * @return array
     */
    public function register_connection($connections)
    {
        $connections[self::$connectionName] = __('Registered Users', 'mailoptin');

        return $connections;
    }

    /**
     * @param int $email_campaign_id
     * @param int $campaign_log_id
     * @param string $subject
     * @param string $content_html
     * @param string $content_text
     *
     * @throws \Exception
     *
     * @return array
     */
    public function send_newsletter($email_campaign_id, $campaign_log_id, $subject, $content_html, $content_text)
    {
        // array of username and email object.
        $users_data = get_users(['fields' => ['user_login', 'user_email']]);

        // campaign log and email campaign IDs to each $users_data
        $users_data = array_reduce($users_data, function ($carry, $user_data) use ($email_campaign_id, $campaign_log_id) {
            $user_data->email_campaign_id = $email_campaign_id;
            $user_data->campaign_log_id = $campaign_log_id;
            $carry[] = $user_data;

            return $carry;
        });

        foreach ($users_data as $user_data) {
            $this->bg_process_instance->push_to_queue($user_data);
        }

        $this->bg_process_instance->save()->dispatch();

        return ['status' => true];
    }


    /**
     * Fulfill interface contract.
     *
     * {@inheritdoc}
     */
    public function replace_placeholder_tags($content, $type = 'html')
    {
        return $this->replace_footer_placeholder_tags($content);
    }

    /**
     * Fulfill interface contract.
     */
    public function get_email_list()
    {
    }

    /**
     * Fulfill interface contract.
     */
    public function subscribe($email, $name, $list_id, $extras = null)
    {
    }

    /**
     * Singleton poop.
     *
     * @return Connect|null
     */
    public static function get_instance()
    {
        static $instance = null;

        if (is_null($instance)) {
            $instance = new self();
        }

        return $instance;
    }
}